@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: apop.cc,v $
-// Revision      : $Revision: 1.2 $
-// Revision date : $Date: 2004/10/13 12:52:59 $
+// Revision      : $Revision: 1.14 $
+// Revision date : $Date: 2005/03/29 11:57:32 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,9 +29,7 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
-#ifdef HAVE_CONFIG_H
-#   include <config.h>
-#endif
+#include "support.h"
 
 #ifdef HAVE_CRYPTO
 #  include <openssl/md5.h>
@@ -44,67 +42,71 @@
 #include "ui-authentication.h"
 #include "socket.h"
 #include "apop.h"
-#include "nls.h"
-
 
+// ========================================================================
+//  base
+// ========================================================================	
+/**
+ * Constructor. The mailbox for the APOP protocol is created from scratch.
+ *
+ * @param biff Pointer to the instance of Gnubiff.
+ */
 Apop::Apop (Biff *biff) : Pop (biff)
 {
-	protocol_ = PROTOCOL_APOP;
+	value ("protocol", PROTOCOL_APOP);
 }
 
+/**
+ * Constructor. The mailbox for the APOP protocol is created by taking the
+ * attributes of the existing mailbox {\em other}.
+ *
+ * @param other Mailbox from which the attributes are taken.
+ */
 Apop::Apop (const Mailbox &other) : Pop (other)
 {
-	protocol_ = PROTOCOL_APOP;
+	value ("protocol", PROTOCOL_APOP);
 }
 
+/// Destructor
 Apop::~Apop (void)
 {
-	delete socket_;
 }
 
-int
-Apop::connect (void)
+// ========================================================================
+//  main
+// ========================================================================	
+/**
+ * A connection to the mailbox is established. If this can't be done then a
+ * {\em pop_socket_err} is thrown. Otherwise gnubiff logins.
+ *
+ * @exception pop_command_err
+ *                     This exception is thrown when we get an unexpected
+ *                     response or if the encrypted login line cannot be
+ *                     created.
+ * @exception pop_socket_err
+ *                     This exception is thrown if a network error occurs.
+ */
+void 
+Apop::connect (void) throw (pop_err)
 {
 	std::string line;
 
-	// Check password is not empty
-	if (password_.empty())
-		ui_authentication_->select (this);
+	// Open the socket
+	Pop::connect ();
 
-	if (password_.empty()) {
-		socket_->status(SOCKET_STATUS_ERROR);
-		status_ = MAILBOX_ERROR;
-		g_warning (_("[%d] Empty password"), uin_);
-		return 0;
-	}
-
-	// Connection
-	if (!socket_->open (hostname_, port_, use_ssl_, certificate_)) {
-		socket_->status (SOCKET_STATUS_ERROR);
-		status_ = MAILBOX_ERROR;
-		return 0;
-	}
-
-	// Does server supports apop protocol ?
+	// Does the server support the apop protocol?
 	//  if so, answer should be something like:
 	//  +OK POP3 server ready <1896.697170952@dbc.mtview.ca.us>
-	if (!socket_->read (line)) return 0;
-	if (line.find ("<") == std::string::npos) {
-		g_warning (_("[%d] Your pop server does not seem to accept apop protocol (no timestamp provided)"), uin_);
-		socket_->status (SOCKET_STATUS_ERROR);
-		status_ = MAILBOX_ERROR;
-		return 0;
+	readline (line);
+	std::string::size_type lt = line.find ("<"), gt = line.find (">");
+	if ((lt == std::string::npos) || (gt == std::string::npos) || (gt < lt)) {
+		g_warning (_("[%d] Your pop server does not seem to accept apop "
+					 "protocol (no timestamp provided)"), uin());
+		throw pop_command_err ();
 	}
 
-	
-
 	// Get time stamp from server
-	std::string timestamp = line.substr (line.find ("<"));
-	timestamp = timestamp.substr (0, timestamp.find (">")+1);
-
-#ifdef DEBUG
-	g_message ("[%d] timestamp is %s", uin_, timestamp.c_str());
-#endif
+	std::string timestamp = line.substr (lt, gt-lt+1);
 
 	// Build message if MD5 library available
 	char hex_response[33];
@@ -113,20 +115,18 @@ Apop::connect (void)
 	MD5_CTX ctx;
 	MD5_Init (&ctx);
 	MD5_Update (&ctx, timestamp.c_str(), timestamp.size());
-	MD5_Update (&ctx, password_.c_str(), password_.size());
+	MD5_Update (&ctx, password().c_str(), password().size());
 	MD5_Final (response, &ctx);
 	for (guint i = 0; i < 16; i++)
 		sprintf (&hex_response[i*2], "%02x", response[i]);
 	hex_response[32] = '\0';
 #else
-	g_message (_("[%d] Problem with crypto that should have been detected at configure time"), uin_);
-	return 0;
+	g_warning (_("[%d] Problem with crypto that should have been detected "
+				 "at configure time"), uin());
+	throw pop_command_err ();
 #endif
 
 	// LOGIN
-	line = "APOP " + username_ + std::string(" ") + std::string (hex_response) + std::string ("\r\n");
-	if (!socket_->write (line)) return 0;
-	if (!socket_->read (line)) return 0;
-
-	return 1;
+	sendline ("APOP " + username() + " " + std::string (hex_response));
+	readline (line); // +OK response
 }
