@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: gui.cc,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.5 $
+// Revision date : $Date: 2005/03/31 16:05:18 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,17 +29,18 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
+#include "support.h"
+
 #include <glade/glade.h>
 #include <gmodule.h>
 #include "gtk_image_animation.h"
-#include "nls.h"
 #include "gui.h"
 
 
 /**
  * Local variable for animation preview in gtk_file_chooser
  **/
-GtkImageAnimation *preview_animation = 0;
+GtkImageAnimation *preview_animation = NULL;
 
 
 /**
@@ -50,38 +51,59 @@ extern "C" {
 								  GdkEvent *event,
 								  gpointer data)
 	{
-		return ((GUI *)data)->on_delete (widget, event);
+		if (data)
+			return ((GUI *)data)->on_delete (widget, event);
+		else
+			unknown_internal_error ();
+		return false;
 	}
 
 	gboolean GUI_on_destroy_event (GtkWidget *widget,
 								   GdkEvent *event,
 								   gpointer data)
 	{
-		return ((GUI *)data)->on_destroy (widget, event);
+		if (data)
+			return ((GUI *)data)->on_destroy (widget, event);
+		else
+			unknown_internal_error ();
+		return false;
+
 	}
 
 	void GUI_on_ok (GtkWidget *widget,
 					gpointer data)
 	{
-		((GUI *)data)->on_ok (widget);
+		if (data)
+			((GUI *)data)->on_ok (widget);
+		else
+			unknown_internal_error ();
 	}
 
 	void GUI_on_apply (GtkWidget *widget,
 						 gpointer data)
 	{
-		((GUI *)data)->on_apply (widget);
+		if (data)
+			((GUI *)data)->on_apply (widget);
+		else
+			unknown_internal_error ();
 	}
 
 	void GUI_on_close (GtkWidget *widget,
 					   gpointer data)
 	{
-		((GUI *)data)->on_close (widget);
+		if (data)
+			((GUI *)data)->on_close (widget);
+		else
+			unknown_internal_error ();
 	}
 
 	void GUI_on_cancel (GtkWidget *widget,
 						gpointer data)
 	{
-		((GUI *)data)->on_cancel (widget);
+		if (data)
+			((GUI *)data)->on_cancel (widget);
+		else
+			unknown_internal_error ();
 	}
 
 	/*
@@ -185,9 +207,40 @@ GUI::create (void)
 	}
 
 	glade_xml_signal_autoconnect_full (xml_, GUI_connect, this);
+	create_insert_version ();
 	return true;
 }
 
+/**
+ *  Insert version and date information into certain widgets. For widgets
+ *  called "gnubiff" (if existing) the following substitutions are made:
+ *  "%v" is substituted by the version number, "%c" is substituted by "CVS" if
+ *  it's a cvs version of gnubiff and "" otherwise..
+ */
+void 
+GUI::create_insert_version (void)
+{
+	static const gchar *widgets[] = {"gnubiff", NULL};
+	static const std::string chars = "vc";
+	std::vector<std::string> toinsert (2);
+	toinsert[0] = std::string (PACKAGE_VERSION);
+	toinsert[1] = std::string (IS_CVS_VERSION ? "CVS" : "");
+	guint i = 0;
+	while (widgets[i] != NULL) {
+		// Don't use GUI::get() because we don't want warning messages if
+		// the widget doesn't exist
+		GtkLabel *label = GTK_LABEL(glade_xml_get_widget (xml_, widgets[i++]));
+		if (!label)
+			continue;
+		// Substitute
+		const gchar *text = gtk_label_get_label (label);
+		if (!text)
+			continue;
+		std::string newtext = gb_substitute (text, chars, toinsert);
+		gtk_label_set_label (label, newtext.c_str());
+	}
+}
+
 void
 GUI::show (std::string name)
 {
@@ -208,7 +261,8 @@ GUI::get (std::string name)
 	GtkWidget *widget = glade_xml_get_widget (xml_, (gchar *) name.c_str());
 	if (!widget)
 		g_warning (_("Cannot find the specified widget (\"%s\")"
-					 " within xml structure (\"%s\")"), name.c_str(), filename_.c_str());
+					 " within xml structure (\"%s\")"), name.c_str(),
+				   filename_.c_str());
 	return widget;
 }
 
@@ -241,7 +295,7 @@ GUI::browse (std::string title,
 		gtk_entry_set_text (GTK_ENTRY (get(widget_name.c_str())), filename);
 		g_free (filename);
 	}
-	if (preview)
+	if (preview && preview_animation)
 		preview_animation->stop();
 	gtk_widget_destroy (chooser);
 
