@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: mh.cc,v $
-// Revision      : $Revision: 1.3 $
-// Revision date : $Date: 2004/10/13 17:16:11 $
+// Revision      : $Revision: 1.10 $
+// Revision date : $Date: 2005/04/01 13:24:49 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -33,42 +33,44 @@
 #include <sstream>
 #include <sys/stat.h>
 #include <utime.h>
+
 #include "mh.h"
 
-
-Mh::Mh (Biff *biff) : Mailbox (biff)
+// ========================================================================
+//  base
+// ========================================================================	
+Mh::Mh (Biff *biff) : Local (biff)
 {
-	protocol_ = PROTOCOL_MH;
+	value ("protocol", PROTOCOL_MH);
 }
 
-Mh::Mh (const Mailbox &other) : Mailbox (other)
+Mh::Mh (const Mailbox &other) : Local (other)
 {
-	protocol_ = PROTOCOL_MH;
+	value ("protocol", PROTOCOL_MH);
 }
 
 Mh::~Mh (void)
 {
 }
 
+
+// ========================================================================
+//  base
+// ========================================================================	
 int
 Mh::connect (void)
 {
 	// Build filename (.mh_sequences)
-	std::string filename;
-	gchar *base=g_path_get_basename(location_.c_str());
-	if (base==std::string(".mh_sequences"))
-		filename=location_;
-	else
-	{
-		gchar *tmp=g_build_filename(location_.c_str(),".mh_sequences",NULL);
-		filename=std::string(tmp);
-		g_free(tmp);
-	}
-	g_free(base);
+	gchar *filename = g_build_filename (address().c_str(), ".mh_sequences",
+										NULL);
+	if (!filename)
+		return false;
 
+	// Open file
 	std::ifstream file;
-	file.open (filename.c_str());
-	if (!file.is_open())
+	file.open (filename);
+	g_free (filename);
+	if (!file.is_open ())
 		return false;
 
 	//  Parse mh sequences and try to find unseen sequence
@@ -76,19 +78,21 @@ Mh::connect (void)
 		std::string line;
 		getline (file, line);
 
-		// Got it !
+		// Got it!
 		if (line.find("unseen:") != std::string::npos) {
 			saved_.clear();
 
 			// Analyze of the unseen sequence  whick looks like:
 			//  unseen: 1-3, 7, 9-13, 16, 19
-			guint i = line.find("unseen:")+std::string("unseen:").size();
+			// Note: size of "unseen:" is 7
+			std::string::size_type i = line.find ("unseen:") + 7;
 
-			// Start parsing line looking for digit, range indicator, number separator
+			// Start parsing line looking for digit, range indicator,
+			// number separator
 			guint inf_bound = 0;
 			guint sup_bound = 0;
 
-			while (i<line.size()) {
+			while (i < line.size()) {
 				// Got a digit ?
 				if (isdigit (line[i])) {
 					do {
@@ -118,7 +122,7 @@ Mh::connect (void)
 						inf_bound = 0;
 					}
 					i++;
-				}	    
+				}
 			}
 			if (inf_bound > 0)
 				saved_.push_back (inf_bound);
@@ -132,50 +136,24 @@ Mh::connect (void)
 }
 
 void
-Mh::get_status (void)
-{
-	std::vector<guint> buffer = saved_;
-
-	status_ = MAILBOX_CHECKING;
-
-	// Parse unseen sequence
-	if (!connect()) {
-		status_ = MAILBOX_ERROR;
-		return;
-	}
-
-
-	// Find mailbox status by comparing saved list with the new one
-	if (saved_.empty()) {
-		status_ = MAILBOX_EMPTY;
-	}
-	else if (buffer == saved_) {
-		status_ = MAILBOX_OLD;
-	}
-	else {
-		status_ = MAILBOX_OLD;
-		guint i, j;
-		for (i=0; i<saved_.size(); i++) {
-			for (j=0; j<buffer.size(); j++) {
-				if (saved_[i] == buffer[j])
-					break;
-			}
-			if (j == buffer.size()) {
-				status_ = MAILBOX_NEW;
-				break;
-			}
-		}
-	}     
-}
-
-void
-Mh::get_header (void)
+Mh::fetch (void)
 {
 	std::vector<std::string> mail;
 
-	new_unread_.clear();
-	new_seen_.clear();
-	for (guint i=0; (i<saved_.size()) && (new_unread_.size() < (unsigned int)(biff_->max_mail_)); i++) {
+	std::vector<guint> buffer = saved_;
+
+	// Parse unseen sequence
+	if (!connect()) {
+		status (MAILBOX_ERROR);
+		return;
+	}
+
+	// Get maximum number of mails to catch
+	guint maxnum = INT_MAX;
+	if (biff_->value_bool ("use_max_mail"))
+		maxnum = biff_->value_uint ("max_mail");
+
+	for (guint i=0; (i<saved_.size()) && (new_unread_.size() < maxnum); i++) {
 		std::string line;
 		std::ifstream file;    
 		std::stringstream s;
@@ -183,11 +161,10 @@ Mh::get_header (void)
 
 		mail.clear();
 
-		gchar *filename=g_build_filename(location_.c_str(),
-										 s.str().c_str(),NULL);
+		gchar *filename = g_build_filename (address().c_str(), s.str().c_str(),
+											NULL);
 		file.open (filename);
-        g_free(filename);
-
+        g_free (filename);
 		if (file.is_open()) {
 			while (!file.eof()) {
 				getline(file, line);
@@ -197,10 +174,4 @@ Mh::get_header (void)
 			file.close();
 		}
 	}
-
-	if ((unread_ == new_unread_) && (new_unread_.size() > 0))
-		status_ = MAILBOX_OLD;
-
-	unread_ = new_unread_;
-	seen_ = new_seen_;
 }
