@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: pop3.cc,v $
-// Revision      : $Revision: 1.2 $
-// Revision date : $Date: 2004/10/29 14:58:47 $
+// Revision      : $Revision: 1.14 $
+// Revision date : $Date: 2005/03/29 11:57:32 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,6 +29,8 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
+#include "support.h"
+
 #include <string>
 #include <sstream>
 #include <glib.h>
@@ -38,69 +40,71 @@
 #include "ui-authentication.h"
 #include "pop3.h"
 #include "socket.h"
-#include "nls.h"
-
 
+// ========================================================================
+//  base
+// ========================================================================	
+/**
+ * Constructor. The mailbox for the POP3 protocol is created from scratch.
+ *
+ * @param biff Pointer to the instance of Gnubiff.
+ */
 Pop3::Pop3 (Biff *biff) : Pop (biff)
 {
-	protocol_ = PROTOCOL_POP3;
+	value ("protocol", PROTOCOL_POP3);
 }
 
+/**
+ * Constructor. The mailbox for the POP3 protocol is created by taking the
+ * attributes of the existing mailbox {\em other}.
+ *
+ * @param other Mailbox from which the attributes are taken.
+ */
 Pop3::Pop3 (const Mailbox &other) : Pop (other)
 {
-	protocol_ = PROTOCOL_POP3;
+	value ("protocol", PROTOCOL_POP3);
 }
 
+/// Destructor
 Pop3::~Pop3 (void)
 {
-	delete socket_;
 }
 
 
-int
-Pop3::connect (void)
+// ========================================================================
+//  main
+// ========================================================================	
+/**
+ * A connection to the mailbox is established. If this can't be done then a
+ * {\em pop_socket_err} is thrown. Otherwise gnubiff logins.
+ *
+ * @exception pop_command_err
+ *                     This exception is thrown when we get an unexpected
+ *                     response.
+ * @exception imap_socket_err
+ *                     This exception is thrown if a network error occurs.
+ */
+void 
+Pop3::connect (void) throw (pop_err)
 {
-	if (biff_->no_clear_password_ && !use_ssl_) {
-		status_  = MAILBOX_BLOCKED;
-		return 0;
-	}
-
-	// show authentication if password is empty
-	if (password_.empty())
-		ui_authentication_->select (this);
-
-	// if it is still empty after authentication, just return
-	if (password_.empty()) {
-		socket_->status (SOCKET_STATUS_ERROR);
-		status_  = MAILBOX_ERROR;
-		return 0;
-	}
-
-	// connection
-	if (!socket_->open (hostname_, port_, use_ssl_, certificate_)) {
-		socket_->status (SOCKET_STATUS_ERROR);
-		status_ = MAILBOX_ERROR;
-		return 0;
-	}
-
 	std::string line;
-	if (!socket_->read (line)) return 0;
 
-	// LOGIN : username
-	line = "USER " + username_ + std::string ("\r\n");
-	if (!socket_->write (line)) return 0;
-	if (!socket_->read (line)) return 0;
+	// Open the socket
+	Pop::connect ();
 
-	// LOGIN : password
-	line = "PASS " + password_ + std::string ("\r\n");
+	readline (line); // +OK response
 
+	// LOGIN: username
+	sendline ("USER " + username());
+	readline (line); // +OK response
+
+	// LOGIN: password
+	sendline ("PASS " + password(), false);
+	readline (line); // +OK response
 #ifdef DEBUG
 	// Just in case someone sends me the output: password won't be displayed
-	g_print ("** Message: [%d] SEND(%s:%d): %s", uin_, hostname_.c_str(), port_, "PASS (hidden)\r\n");
+	std::string line_no_password = "PASS (hidden)\r\n";
+	g_message ("[%d] SEND(%s:%d): %s", uin(), address().c_str(), port(),
+			   line_no_password.c_str());
 #endif
-
-	if (!socket_->write (line,false)) return 0;
-	if (!socket_->read (line)) return 0;
-
-	return 1;
 }
