@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: pop3.h,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.5 $
+// Revision date : $Date: 2005/03/29 11:57:32 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -34,17 +34,23 @@
 
 #include "pop.h"
 
-
+/**
+ * Mailbox for the POP3 protocol. 
+ */
 class Pop3 : public Pop {
 
 public:
-	/* base */
+	// ========================================================================
+	//  base
+	// ========================================================================	
 	Pop3 (class Biff *biff);
 	Pop3 (const Mailbox &other);
 	~Pop3 (void);
 
-	/* main */
-	int connect (void);
+	// ========================================================================
+	//  main
+	// ========================================================================	
+	void connect (void) throw (pop_err);
 };
 
 #endif
