@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: socket.cc,v $
-// Revision      : $Revision: 1.6 $
-// Revision date : $Date: 2004/11/14 22:49:01 $
+// Revision      : $Revision: 1.29 $
+// Revision date : $Date: 2005/03/20 13:29:15 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,6 +29,8 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
+#include "support.h"
+
 #include <sstream>
 #include <cstdio>
 #include <unistd.h>
@@ -36,6 +38,7 @@
 #include <netdb.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/time.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
@@ -45,8 +48,18 @@
 #include "nls.h"
 
 
+// ========================================================================
+//  Static features
+// ========================================================================	
 GStaticMutex Socket::hostname_mutex_  = G_STATIC_MUTEX_INIT;
+#ifdef HAVE_LIBSSL
+Certificate *Socket::ui_cert_ = 0;
+GStaticMutex Socket::ui_cert_mutex_ = G_STATIC_MUTEX_INIT;
+#endif
 
+// ========================================================================
+//  base
+// ========================================================================	
 Socket::Socket (Mailbox *mailbox)
 {
 	mailbox_ = mailbox;
@@ -66,7 +79,10 @@ Socket::Socket (Mailbox *mailbox)
 	SSL_load_error_strings();
 	context_ = SSL_CTX_new (SSLv23_client_method());
 	bypass_certificate_ = false;
-	ui_certificate_ = new Certificate ();
+	g_static_mutex_lock (&ui_cert_mutex_);
+	if (ui_cert_ == 0)
+		ui_cert_ = new Certificate ();
+	g_static_mutex_unlock (&ui_cert_mutex_);
 #endif
 }
 
@@ -79,14 +95,19 @@ Socket::~Socket (void)
 gint
 Socket::open (std::string hostname,
 			  gushort port,
-			  gboolean use_ssl,
-			  std::string certificate)
+			  guint authentication,
+			  std::string certificate,
+			  guint timeout)
 {
 	hostname_ = hostname;
 	port_ = port;
-	use_ssl_ = use_ssl;
+	if ((authentication == AUTH_SSL) || (authentication == AUTH_CERTIFICATE))
+		use_ssl_ = true;
 	certificate_ = certificate;
 
+	// Get options' values to avoid periodic lookups
+	prevdos_line_length_=mailbox_->biff()->value_uint ("prevdos_line_length");
+
 	struct sockaddr_in sin;
 	struct hostent *host;
 	struct in_addr address;
@@ -105,6 +126,13 @@ Socket::open (std::string hostname,
 		return 0;
 	}
 
+	// Set non-blocking socket if timeout required
+	if (timeout > 0) {
+		int arg = fcntl (sd_, F_GETFL, NULL);
+		arg |= O_NONBLOCK;
+		fcntl (sd_, F_SETFL, arg);
+	}
+
 	// Setting socket info for connection
 	memset ((char *)&sin, 0, sizeof(sin));
 	sin.sin_family = AF_INET;
@@ -133,12 +161,54 @@ Socket::open (std::string hostname,
 	else
 		sin.sin_addr = address;
 
-	// Initiate the connection on the socket
-	if ((connect (sd_, (struct sockaddr *) &sin, sizeof (struct sockaddr_in))) == -1) {
+
+	int res = connect (sd_, (struct sockaddr *) &sin, sizeof (struct sockaddr_in));
+	if ((timeout) && (res < 0)) {
+		if (errno == EINPROGRESS) {
+			int valopt;
+			struct timeval tv;
+			tv.tv_sec = timeout;
+			tv.tv_usec = 0;
+			fd_set myset;
+			FD_ZERO (&myset);
+			FD_SET (sd_, &myset);
+			if (select (sd_+1, NULL, &myset, NULL, &tv) > 0) {
+				socklen_t lon = sizeof (int);
+				getsockopt (sd_, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon);
+				if (valopt) {
+					g_static_mutex_unlock (&hostname_mutex_);
+					::close (sd_);
+					sd_ = SD_CLOSE;
+					g_warning (_("[%d] Unable to connect to %s on port %d"), uin_, hostname_.c_str(), port_);
+					return 0;
+				}
+			}
+			else {
+				g_static_mutex_unlock (&hostname_mutex_);
+				::close (sd_);
+				sd_ = SD_CLOSE;
+				g_warning (_("[%d] Unable to connect to %s on port %d"), uin_, hostname_.c_str(), port_);
+				return 0;
+			}
+		}
+		else {
+			g_static_mutex_unlock (&hostname_mutex_);
+			::close (sd_);
+			sd_ = SD_CLOSE;
+			g_warning (_("[%d] Unable to connect to %s on port %d"), uin_, hostname_.c_str(), port_);
+			return 0;
+		}
+		// Set to blocking mode again...
+		int arg = fcntl (sd_, F_GETFL, NULL);
+		arg &= (~O_NONBLOCK);
+		fcntl (sd_, F_SETFL, arg);
+	}
+	else if (res == -1) {
 		g_static_mutex_unlock (&hostname_mutex_);
 		::close (sd_);
 		sd_ = SD_CLOSE;
-		g_warning (_("[%d] Unable to connect to %s on port %d"), uin_, hostname_.c_str(), port_);
+		g_warning (_("[%d] Unable to connect to %s on port %d"), uin_,
+				   hostname_.c_str(), port_);
 		return 0;
 	}
 
@@ -146,9 +216,15 @@ Socket::open (std::string hostname,
 
 #ifdef HAVE_LIBSSL
 	if (use_ssl_) {
-		if (certificate_.size() > 0){
-			if (!SSL_CTX_load_verify_locations (context_, certificate_.c_str(), NULL)) {
-				g_warning(_("[%d] Failed to load certificate (%s) for %s"), uin_, certificate_.c_str(), hostname_.c_str());
+		if (certificate_.size() > 0) {
+			const gchar *capath = mailbox_->biff()->value_gchar ("dir_certificates");
+			if (*capath == '\0')
+				capath = NULL;
+			if (!SSL_CTX_load_verify_locations (context_, certificate_.c_str(),
+												capath)) {
+				g_warning(_("[%d] Failed to load certificate (%s) for %s"),
+						  uin_, hostname_.c_str(), certificate_.c_str(),
+						  hostname_.c_str());
 				::close (sd_);
 				sd_ = SD_CLOSE;
 				return 0;
@@ -158,12 +234,12 @@ Socket::open (std::string hostname,
 		else
 			SSL_CTX_set_verify (context_, SSL_VERIFY_NONE, NULL);
     
-
 		ssl_ = SSL_new (context_);
 		if ((!ssl_) || (SSL_set_fd (ssl_, sd_) == 0)) {
 			::close (sd_);
 			sd_ = SD_CLOSE;
-			g_warning (_("[%d] Unable to connect to %s on port %d"), uin_, hostname_.c_str(), port_);
+			g_warning (_("[%d] Unable to connect to %s on port %d"), uin_,
+					   hostname_.c_str(), port_);
 			return 0;
 		}
 		if (SSL_connect (ssl_) != 1) {
@@ -176,7 +252,9 @@ Socket::open (std::string hostname,
 		}
 
 		if ((certificate_.size() > 0) && (SSL_get_verify_result(ssl_) != X509_V_OK)) {
-			ui_certificate_->select (this);
+			g_static_mutex_lock (&ui_cert_mutex_);
+			ui_cert_->select (this);
+			g_static_mutex_unlock (&ui_cert_mutex_);
 			if (!bypass_certificate_) {
 				SSL_free (ssl_);
 				ssl_ = NULL;
@@ -192,135 +270,165 @@ Socket::open (std::string hostname,
 	return 1;
 }
 
-
-gint
+/**
+ *  Close the socket.
+ */
+void 
 Socket::close (void)
 {
+	// Is socket already closed?
+	if (sd_ == SD_CLOSE) {
+#ifdef HAVE_LIBSSL
+		if (ssl_) {
+			SSL_free (ssl_);
+			ssl_ = NULL;
+		}
+#endif
+		return;
+	}
+
 #ifdef DEBUG
 	g_message ("[%d] CLOSE %s:%d", uin_, hostname_.c_str(), port_);
 #endif
-	std::string line;  
-	if (sd_ != SD_CLOSE) {
-		fcntl (sd_, F_SETFL, O_NONBLOCK);
-		do {
-			read (line, false, false);
-		} while (!line.empty());
-	}
-  
+
+	// Read lines that are not yet read
+	std::string line;
+	fcntl (sd_, F_SETFL, O_NONBLOCK);
+	guint cnt = 1 + mailbox_->biff()->value_uint ("prevdos_close_socket");
+	do {
+		read (line, false, false);
+	} while ((!line.empty()) && (cnt--));
+
 #ifdef HAVE_LIBSSL
-	if (use_ssl_ && ssl_) {
-		if (sd_ != SD_CLOSE)
-			SSL_shutdown (ssl_);
+	if (ssl_) {
+		SSL_shutdown (ssl_);
 		SSL_free (ssl_);
-		ssl_ = 0;
+		ssl_ = NULL;
 	}
 #endif
-	if(sd_ != SD_CLOSE)
-		::close (sd_);
+	::close (sd_);
 	sd_ = SD_CLOSE;
-	return 1;
 }
 
-
 gint
-Socket::write (std::string line,
-			   gboolean debug)
+Socket::write (std::string line, gboolean print)
 {
+	// Do not allow writing to a closed, or invalid socket. This causes
+	// SEGV in SSL.
+	if (sd_ == SD_CLOSE)
+		return SOCKET_STATUS_ERROR;
+
 	status_ = -1;
 
+	// TEMP_FAILURE_RETRY will re-call the method if the write primitive
+	// is interrupted by a signal.
+	
 #ifdef HAVE_LIBSSL
 	if (use_ssl_) {
-		if (SSL_write (ssl_, line.c_str(), line.size()) <= 0)
+		if (TEMP_FAILURE_RETRY(SSL_write (ssl_, line.c_str(), line.size())) <= 0)
 			status_ = SOCKET_STATUS_ERROR;
 		else
 			status_ = SOCKET_STATUS_OK;
 	}
 #endif
 	if (status_ == -1) {
-		if (::write (sd_, line.c_str(), line.size()) <= 0)
+		if (TEMP_FAILURE_RETRY(::write (sd_, line.c_str(), line.size())) <= 0)
 			status_ = SOCKET_STATUS_ERROR;
 		else
 			status_ = SOCKET_STATUS_OK;
 	}
 
 #ifdef DEBUG
-	if (debug)
-		g_print ("** Message: [%d] SEND(%s:%d): %s", uin_, hostname_.c_str(), port_, line.c_str());
+	if (print)
+		g_message ("[%d] SEND(%s:%d): %s", uin_, hostname_.c_str(), port_,
+				   line.substr(0, line.size()-2).c_str());
 #endif
 
-	if (!status_) {
+	if ((print) && (status_ != SOCKET_STATUS_OK)) {
 		g_warning (_("[%d] Unable to write to %s on port %d"), uin_, hostname_.c_str(), port_);
 		close();
-		mailbox_->status (MAILBOX_ERROR);
 	}
 
 	return status_;
 }
 
-gint
-Socket::read (std::string &line,
-			  gboolean debug,
-			  gboolean check)
+gint 
+Socket::read (std::string &line, gboolean print, gboolean check)
 {
+	// Do not allow writing to a closed, or invalid socket. This causes
+	// SEGV in SSL.
+	if (sd_ == SD_CLOSE)
+		return SOCKET_STATUS_ERROR;
+
 	char buffer;
-	int status=0;
+	int status = 0;
 	line = "";
 	status_ = -1;
 
-	gint cnt=1+preventDoS_lineLength_; 
+	gint cnt = 1 + prevdos_line_length_;
+
+	// TEMP_FAILURE_RETRY will re-call the method if the read primitive
+	// is interrupted by a signal.
+	
+	errno = 0;
 #ifdef HAVE_LIBSSL
 	if (use_ssl_) {
-		while ((0<cnt--) && ((status = SSL_read (ssl_, &buffer, 1)) > 0)
-			     && (buffer != '\n'))
+		while ((0<cnt--)
+			   && ((status=TEMP_FAILURE_RETRY(SSL_read (ssl_, &buffer, 1)))>0)
+			   && (buffer != '\n'))
 			line += buffer;
-		if ((status > 0) && (cnt>=0))
-			status_ = SOCKET_STATUS_OK;
-		else
-			status_ = SOCKET_STATUS_ERROR;
 	}
+	else
 #endif
-	if (status_ == -1) {
-
-		while ((0<cnt--) && ((status = ::read (sd_, &buffer, 1)) > 0)
-			     && (buffer != '\n'))
+	{
+		while ((0<cnt--)
+			   && ((status=TEMP_FAILURE_RETRY(::read (sd_, &buffer, 1))) > 0)
+			   && (buffer != '\n'))
 			line += buffer;
-		if ((status > 0) && (cnt>=0))
-			status_ = SOCKET_STATUS_OK;
-		else
-			status_ = SOCKET_STATUS_ERROR;
 	}
 
+	if (errno == EAGAIN)
+		status_ = SOCKET_TIMEOUT;
+	else if ((status > 0) && (cnt>=0))
+		status_ = SOCKET_STATUS_OK;
+	else
+		status_ = SOCKET_STATUS_ERROR;
+	
 	if (!check)
 		return status_;
 
 #ifdef DEBUG
-	if (debug)
-		g_message ("[%d] RECV(%s:%d): %s", uin_, hostname_.c_str(), port_, line.c_str());
+	if (print)
+		if (status_ == SOCKET_TIMEOUT)
+			g_message ("[%d] RECV TIMEOUT(%s:%d)", uin_, hostname_.c_str(), port_);
+		else if (status_ == SOCKET_STATUS_OK)
+			g_message ("[%d] RECV(%s:%d): %s", uin_, hostname_.c_str(), port_, line.c_str());
+		else
+			g_message ("[%d] RECV ERROR(%s:%d): %s", uin_, hostname_.c_str(), port_, strerror(status));
 #endif
-	if (!status_) {
+	if (status_ == SOCKET_STATUS_ERROR) {
 		g_warning (_("[%d] Unable to read from %s on port %d"), uin_, hostname_.c_str(), port_);
 		close();
-		mailbox_->status (MAILBOX_ERROR);
 	}
 
-	// Check imap4
-	if ((mailbox_->protocol() == PROTOCOL_IMAP4) && status_) {
-		if (line.find ("* BYE") == 0) {
-			close();
-			status_ = SOCKET_STATUS_ERROR;
-			mailbox_->status (MAILBOX_ERROR);
-		}
-	}
-
-	// Check pop
-	if (((mailbox_->protocol() == PROTOCOL_APOP) || 
-		 (mailbox_->protocol() == PROTOCOL_POP3)) && status_) {
-		 if (line.find ("-ERR") == 0) {
-			 close();
-			 status_ = SOCKET_STATUS_ERROR;
-			 mailbox_->status (MAILBOX_ERROR);
-		 }
-	}
-	
 	return status_;
 }
+
+/**
+ * Specify a timeout value for read operations on this socket.  Read
+ * operations will block until the given time has expired.  If the
+ * {\em gint Socket::read (std::string, gboolean, gboolean)} method
+ * returns because of the timeout it will return with {\em SOCKET_TIMEOUT}.
+ *
+ * @param  timeout   Time in seconds for timeout duration.
+ */
+void 
+Socket::set_read_timeout(gint timeout)
+{
+	struct timeval tv;
+	tv.tv_sec = timeout;
+	tv.tv_usec = 0;
+	if (setsockopt(sd_, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) == -1)
+		g_warning (_("Could not set read timeout on socket: %s"),
+				   strerror(errno));
+}
