@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: socket.h,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.11 $
+// Revision date : $Date: 2005/03/20 13:29:15 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -43,11 +43,13 @@
 
 #include <glib.h>
 #include <string>
+#include "gnubiff_options.h"
 
-const int	SOCKET_STATUS_OK		=	 1;
-const int	SOCKET_STATUS_ERROR		=	 0;
-const int	SD_CLOSE				=	-1;
 
+const gint	SOCKET_TIMEOUT			=	 2;
+const gint	SOCKET_STATUS_OK		=	 1;
+const gint	SOCKET_STATUS_ERROR		=	 0;
+const gint	SD_CLOSE				=	-1;
 
 class Socket {
 
@@ -58,14 +60,21 @@ protected:
 	guint				uin_;
 	gboolean			use_ssl_;
 	std::string			certificate_;
+	/** Maximum length of a line being read from the socket before assuming
+	 *  a DoS attack. When opening a connection this variable is set by
+	 *  taking the value of the option "prevdos_line_length" (this is done
+	 *  to avoid looking up the option for every line being read). */
+	guint				prevdos_line_length_;
 # ifdef HAVE_LIBSSL
-	SSL_CTX *			context_;
-	SSL *				ssl_;
-	gboolean			bypass_certificate_;
-	class Certificate *	ui_certificate_;
+	SSL_CTX *					context_;
+	SSL *						ssl_;
+	gboolean					bypass_certificate_;
+	static class Certificate *	ui_cert_;
+	static GStaticMutex			ui_cert_mutex_;		// Lock to avoid conflicts
 # endif
 	gint				sd_;
 	gint				status_;
+
 	static GStaticMutex	hostname_mutex_;
 
 
@@ -81,21 +90,23 @@ public:
 	 **/
 	gint open  (std::string hostname = "",
 				gushort port = 0,
-				gboolean use_ssl = false,
-				std::string certificate = "");
-	gint close (void);
-	gint write (std::string line,
-				gboolean debug = true);
+				guint authentication = AUTH_SSL,
+				std::string certificate = "",
+				guint timeout = 5);
+	void close (void);
+	gint write (std::string line, gboolean print = true);
 	gint read  (std::string &line,
-				gboolean debug = true,
+				gboolean print = true,
 				gboolean check = true);
-
+  
 	/**
 	 * access
 	 **/
 	void status (const gint status)				{status_ = status;}
 	gint status (void)							{return status_;}
 	std::string hostname (void)					{return hostname_;}
+	void set_read_timeout(gint timeout);
+
 #ifdef HAVE_LIBSSL
 	SSL *ssl (void)								{return ssl_;}
 	const gboolean bypass_certificate (void)	{return bypass_certificate_;}
@@ -103,4 +114,18 @@ public:
 #endif
 };
 
+/* TEMP_FAILURE_RETRY seems to be available only on Linux. For systems that
+ * don't have this macro we provide our own version. This code was taken from
+ * file "/usr/include/unistd.h" from Debian package "libc6-dev"
+ * version 2.3.2.ds1-20. */
+#ifndef TEMP_FAILURE_RETRY
+#define TEMP_FAILURE_RETRY(expression) \
+	(__extension__ \
+	 ({ long int __result; \
+		do __result = (long int) (expression); \
+		while (__result == -1L && errno == EINTR); \
+		__result; }))
+#endif
+
+
 #endif
