@@ -1,6 +1,6 @@
 // ========================================================================
 // gnubiff -- a mail notification program
-// Copyright (c) 2000-2004 Nicolas Rougier
+// Copyright (c) 2000-2005 Nicolas Rougier
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: ui-applet-gnome.cc,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.6 $
+// Revision date : $Date: 2005/03/21 15:33:09 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,9 +29,7 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
-#ifdef HAVE_CONFIG_H
-#   include <config.h>
-#endif
+#include "support.h"
 
 #include <sstream>
 #include <cstdio>
@@ -52,21 +50,30 @@ extern "C" {
 								GdkEventCrossing *event,
 								gpointer data)
 	{
-		((AppletGnome *) data)->tooltip_update ();
+		if (data)
+			((AppletGnome *) data)->tooltip_update ();
+		else
+			unknown_internal_error ();
 	}
 
 	void APPLET_GNOME_on_change_orient (GtkWidget *widget,
 										PanelAppletOrient orient,
 										gpointer data)
 	{
-		((AppletGnome *) data)->update();
+		if (data)
+			((AppletGnome *) data)->update();
+		else
+			unknown_internal_error ();
 	}
 
 	void APPLET_GNOME_on_change_size (GtkWidget *widget,
 									  int size,
 									  gpointer data)
 	{
-		((AppletGnome *) data)->update ();
+		if (data)
+			((AppletGnome *) data)->update ();
+		else
+			unknown_internal_error ();
 	}
 
 	void APPLET_GNOME_on_change_background (GtkWidget *widget,
@@ -75,58 +82,70 @@ extern "C" {
 											GdkPixmap *pixmap,
 											gpointer data)
 	{
-		((AppletGnome *) data)->update ();
+		if (data)
+			((AppletGnome *) data)->update ();
+		else
+			unknown_internal_error ();
 	}
 
 	gboolean APPLET_GNOME_on_button_press (GtkWidget *widget,
 										   GdkEventButton *event,
 										   gpointer data)
 	{
-		return ((AppletGnome *) data)->on_button_press (event);
+		if (data)
+			return ((AppletGnome *) data)->on_button_press (event);
+		else
+			unknown_internal_error ();
+		return false;
 	}
 
 	void APPLET_GNOME_on_menu_properties (BonoboUIComponent *uic,
 										  gpointer data,
 										  const gchar *verbname)
 	{
-		((AppletGnome *) data)->on_menu_properties (uic, verbname);
+		if (data)
+			((AppletGnome *) data)->on_menu_properties (uic, verbname);
+		else
+			unknown_internal_error ();
 	}
 
-	void APPLET_GNOME_on_menu_mail_app (BonoboUIComponent *uic,
-										gpointer data,
-										const gchar *verbname)
+	void APPLET_GNOME_on_menu_command (BonoboUIComponent *uic,
+									   gpointer data,
+									   const gchar *verbname)
 	{
-		((AppletGnome *) data)->on_menu_mail_app (uic, verbname);
+		if (data)
+			((AppletGnome *) data)->on_menu_command (uic, verbname);
+		else
+			unknown_internal_error ();
 	}
 
 	void APPLET_GNOME_on_menu_mail_read (BonoboUIComponent *uic,
 										 gpointer data,
 										 const gchar *verbname)
 	{
-		((AppletGnome *) data)->on_menu_mail_read (uic, verbname);
-	}
-
-	void APPLET_GNOME_on_menu_about (BonoboUIComponent *uic,
-									 gpointer data,
-									 const gchar *verbname)
-	{
-		((AppletGnome *) data)->on_menu_about (uic, verbname);
+		if (data)
+			((AppletGnome *) data)->on_menu_mail_read (uic, verbname);
+		else
+			unknown_internal_error ();
 	}
 
 	gboolean APPLET_GNOME_reconnect (gpointer data)
 	{
-		g_signal_connect (G_OBJECT (((AppletGnome *)data)->panelapplet()),
-						  "change_background",
-						  GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_background),
-						  data);
-		return FALSE;
+		if (data) {
+			g_signal_connect (G_OBJECT (((AppletGnome *)data)->panelapplet()),
+							  "change_background",
+							  GTK_SIGNAL_FUNC (APPLET_GNOME_on_change_background),
+							  data);
+		}
+		else
+			unknown_internal_error ();
+		return false;
 	}
 }
 
 
 AppletGnome::AppletGnome (Biff *biff) : Applet (biff, GNUBIFF_DATADIR"/applet-gnome.glade")
 {
-	about_ = 0;
 }
 
 AppletGnome::~AppletGnome (void)
@@ -139,9 +158,8 @@ AppletGnome::dock (GtkWidget *applet)
 {
 	static const BonoboUIVerb gnubiffMenuVerbs [] = {
 		BONOBO_UI_VERB ("Props",   APPLET_GNOME_on_menu_properties),
-		BONOBO_UI_VERB ("MailApp", APPLET_GNOME_on_menu_mail_app),
+		BONOBO_UI_VERB ("MailApp", APPLET_GNOME_on_menu_command),
 		BONOBO_UI_VERB ("MailRead", APPLET_GNOME_on_menu_mail_read),
-		BONOBO_UI_VERB ("About",   APPLET_GNOME_on_menu_about), 
 		BONOBO_UI_VERB_END
 	};
  
@@ -159,7 +177,7 @@ AppletGnome::dock (GtkWidget *applet)
 
 	GtkImageAnimation *anim = new GtkImageAnimation (GTK_IMAGE(get("image")));
 	g_object_set_data (G_OBJECT(get("image")), "_animation_", anim);
-	anim->open (biff_->biff_newmail_image_.c_str());
+	anim->open (biff_->value_string ("newmail_image"));
 	anim->start();
 
 	g_signal_connect (G_OBJECT (applet), "enter_notify_event",  GTK_SIGNAL_FUNC (APPLET_GNOME_on_enter), this);
@@ -173,12 +191,14 @@ AppletGnome::dock (GtkWidget *applet)
 }
 
 void
-AppletGnome::update (void)
+AppletGnome::update (gboolean no_popup)
 {
 	// Is there another update going on ?
 	if (!g_mutex_trylock (update_mutex_))
 		return;
 
+	Applet::update (no_popup);
+
 	std::string text;
 	guint unread = unread_markup (text);
 	gtk_label_set_markup (GTK_LABEL(get ("hunread")), text.c_str());
@@ -188,21 +208,21 @@ AppletGnome::update (void)
 	GtkImageAnimation *anim = (GtkImageAnimation *) g_object_get_data (G_OBJECT(get("image")), "_animation_");
 	// Pick image/animation
 	if (unread > 0) {
-		if (!biff_->biff_use_newmail_image_) {
+		if (!biff_->value_bool ("use_newmail_image")) {
 			gtk_widget_hide (get("image"));
 		}
 		else {
 			gtk_widget_show (get("image"));
-			anim->open (biff_->biff_newmail_image_.c_str());
+			anim->open (biff_->value_string ("newmail_image"));
 		}
 	}
 	else {
-		if (!biff_->biff_use_nomail_image_) {
+		if (!biff_->value_bool ("use_nomail_image")) {
 			gtk_widget_hide (get("image"));
 		}
 		else {
 			gtk_widget_show (get("image"));
-			anim->open (biff_->biff_nomail_image_.c_str());
+			anim->open (biff_->value_string ("nomail_image"));
 		}
 	}
 
@@ -211,7 +231,8 @@ AppletGnome::update (void)
 	// The panel is oriented horizontally
 	if ((orient == PANEL_APPLET_ORIENT_DOWN) || (orient == PANEL_APPLET_ORIENT_UP)) {
 		gtk_widget_hide (get ("vunread"));
-		if (((unread > 0) && (biff_->biff_use_newmail_text_)) || ((unread == 0) && (biff_->biff_use_nomail_text_)))
+		if (((unread > 0) && (biff_->value_bool ("use_newmail_text")))
+			|| ((unread == 0) && (biff_->value_bool ("use_nomail_text"))))
 			gtk_widget_show (get ("hunread"));
 		else
 			gtk_widget_hide (get ("hunread"));
@@ -223,7 +244,8 @@ AppletGnome::update (void)
 	// The panel is oriented vertically
 	else {
 		gtk_widget_hide (get ("hunread"));
-		if (((unread > 0) && (biff_->biff_use_newmail_text_)) || ((unread == 0) && (biff_->biff_use_nomail_text_)))
+		if (((unread > 0) && (biff_->value_bool ("use_newmail_text")))
+			|| ((unread == 0) && (biff_->value_bool ("use_nomail_text"))))
 			gtk_widget_show (get ("vunread"));
 		else
 			gtk_widget_hide (get ("vunread"));
@@ -239,8 +261,8 @@ AppletGnome::update (void)
 	GdkColor color;
 	GdkPixmap *pixmap = NULL;
 	type = panel_applet_get_background (PANEL_APPLET(applet_), &color, &pixmap);
-	if (pixmap && G_IS_OBJECT(pixmap)) {
-		GtkStyle* style = gtk_style_copy (gtk_widget_get_style (applet_));	
+	if (pixmap && G_IS_OBJECT(pixmap)) {		
+		GtkStyle* style = gtk_style_copy (gtk_widget_get_style (applet_));
 		style->bg_pixmap[0] = pixmap;
 		gtk_widget_set_style (applet_, style);
 		gtk_widget_set_style (get("table"), style);
@@ -289,20 +311,24 @@ AppletGnome::on_button_press (GdkEventButton *event)
 {
 	// Double left click : start mail app
 	if ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1)) {
-		std::string command = biff_->mail_app_ + " &";
-		system (command.c_str());
+		if ((biff_->value_bool ("use_double_command")) &&
+			 (!biff_->value_string ("double_command").empty ())) {
+			std::string command = biff_->value_string ("double_command")+" &";
+			system (command.c_str());
+		}
 	}
 	// Single left click : popup menu
-	else if (event->button == 1)
-		watch();
+	else if (event->button == 1) {
+		force_popup_ = true;
+		update ();
+	}
 	// Single middle click : mark mails as read
 	else if (event->button == 2) {
 		for (unsigned int i=0; i<biff_->size(); i++)
-			biff_->mailbox(i)->mark_all();
+			biff_->mailbox(i)->read();
 		force_popup_ = true;
 		biff_->popup()->hide();
-		biff_->applet()->process();
-		biff_->applet()->update();
+		update();
 	}
 
 	return FALSE;
@@ -311,18 +337,16 @@ AppletGnome::on_button_press (GdkEventButton *event)
 void
 AppletGnome::on_menu_properties (BonoboUIComponent *uic, const gchar *verbname)
 {
-	biff_->applet()->watch_off();
 	biff_->popup()->hide();
 	biff_->preferences()->show();
-	if (about_ != 0)
-		gdk_window_hide (about_->window);
 }
 
 void
-AppletGnome::on_menu_mail_app (BonoboUIComponent *uic, const gchar *verbname)
+AppletGnome::on_menu_command (BonoboUIComponent *uic, const gchar *verbname)
 {
-	if (!biff_->mail_app_.empty()) {
-		std::string command = biff_->mail_app_ + " &";
+	if ((biff_->value_bool ("use_double_command")) &&
+		 (!biff_->value_string ("double_command").empty ())) {
+		std::string command = biff_->value_string ("double_command") + " &";
 		system (command.c_str());
 	}
 }
@@ -331,34 +355,9 @@ void
 AppletGnome::on_menu_mail_read (BonoboUIComponent *uic, const gchar *verbname)
 {
 	for (unsigned int i=0; i<biff_->size(); i++)
-		biff_->mailbox(i)->mark_all();
+		biff_->mailbox(i)->read();
 	force_popup_ = true;
 	biff_->popup()->hide();
-	biff_->applet()->process();
-	biff_->applet()->update();
+	update();
 }
 
-void
-AppletGnome::on_menu_about (BonoboUIComponent *uic, const gchar *verbname)
-{
-	biff_->popup()->hide();
-	biff_->preferences()->hide();
-
-	static const gchar *authors[] = {
-		"Nicolas Rougier <Nicolas.Rougier@loria.fr>",
-		NULL
-	};
- 
-	if (about_ != 0) {
-		gdk_window_show (about_->window);
-		gdk_window_raise (about_->window);
-		return;
-	}
-	about_ = gnome_about_new ("gnubiff applet", PACKAGE_VERSION,
-							  _("Copyright (c) 2000-2004 Nicolas Rougier\n"),
-							  _("This program is part of the GNU project, released under the aegis of GNU."),
-							  authors,
-							  NULL, NULL, gdk_pixbuf_new_from_file (GNUBIFF_DATADIR"/logo.png", 0));
-	g_signal_connect (G_OBJECT (about_), "destroy", G_CALLBACK (gtk_widget_destroyed), &about_);
-	gtk_widget_show (about_);
-}
