@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: ui-applet-gtk.h,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.3 $
+// Revision date : $Date: 2004/12/04 20:38:22 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -38,19 +38,25 @@
 class AppletGtk : public Applet {
 
  public:
-	/* base */
+	// ========================================================================
+	//  base
+	// ========================================================================	
 	AppletGtk (class Biff *biff);
 	~AppletGtk (void);
 
-	/* main */
-	void update (void);
+	// ========================================================================
+	//  main
+	// ========================================================================	
+	void update (gboolean no_popup = false);
 	gint create (void);
 	void show (std::string name = "dialog");
 	void tooltip_update (void);
 
-	/* callbacks */
+	// ========================================================================
+	//  callbacks
+	// ========================================================================	
 	gboolean on_button_press (GdkEventButton *event);
-	void on_menu_mail_app (void);
+	void on_menu_command (void);
 	void on_menu_mark (void);
 	void on_menu_preferences (void);
 	void on_menu_about (void);
