@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: ui-certificate.cc,v $
-// Revision      : $Revision: 1.2 $
-// Revision date : $Date: 2004/10/14 10:19:45 $
+// Revision      : $Revision: 1.5 $
+// Revision date : $Date: 2004/12/15 08:31:16 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -29,8 +29,10 @@
 // -*- mode:C++; tab-width:4; c-basic-offset:4; indent-tabs-mode:nil -*-
 // ========================================================================
 
+#include "support.h"
+
 #include <sstream>
-#include "nls.h"
+
 #include "ui-certificate.h"
 #include "socket.h"
 
@@ -60,9 +62,7 @@ Certificate::select (Socket *socket)
 {
 	if (socket) {
 		socket_ = socket;
-		gdk_threads_enter();
 		show ();
-		gdk_threads_leave();
 	}
 }
 
@@ -132,7 +132,7 @@ Certificate::on_ok (GtkWidget *widget)
 	socket_->bypass_certificate (true);
 #endif
 	hide();
-	gtk_main_quit ();
+	gtk_main_quit();
 }
 
 void
@@ -144,7 +144,7 @@ Certificate::on_cancel (GtkWidget *widget)
 	socket_->bypass_certificate (false);
 #endif
 	hide();
-	gtk_main_quit ();
+	gtk_main_quit();
 }
 
 gboolean
@@ -157,7 +157,7 @@ Certificate::on_destroy (GtkWidget *widget,
 	socket_->bypass_certificate (false);
 #endif
 	hide();
-	gtk_main_quit ();
+	gtk_main_quit();
 	return TRUE;
 }
 
@@ -171,6 +171,6 @@ Certificate::on_delete	(GtkWidget *widget,
 	socket_->bypass_certificate (false);
 #endif
 	hide();
-	gtk_main_quit ();
+	gtk_main_quit();
 	return TRUE;
 }
