@@ -19,8 +19,8 @@
 // ========================================================================
 //
 // File          : $RCSfile: ui-preferences.h,v $
-// Revision      : $Revision: 1.1 $
-// Revision date : $Date: 2004/10/06 13:21:57 $
+// Revision      : $Revision: 1.11 $
+// Revision date : $Date: 2005/02/06 21:01:16 $
 // Author(s)     : Nicolas Rougier
 // Short         : 
 //
@@ -33,19 +33,22 @@
 #define __UI_PREFERENCES_H__
 
 #include "gui.h"
+#include "ui-properties.h"
 
 enum {
   COLUMN_UIN,
-  COLUMN_MAILBOX_ICON,
+  COLUMN_MAILBOX_STOCK_ID,
   COLUMN_SSL_ICON,
   COLUMN_MAILBOX,
-  COLUMN_FOREGROUND,
   COLUMN_FORMAT,
-  COLUMN_STATUS,
-  COLUMN_POLLTIME,
+  COLUMN_STATUS_STOCK_ID,
+  COLUMN_SECURITY_STOCK_ID,
   N_COLUMNS
 };
 
+enum { COL_EXP_ID, COL_EXP_NAME, COL_EXP_GROUPNAME, COL_EXP_TYPE,
+	   COL_EXP_VALUE, COL_EXP_NAME_ITALIC, COL_EXP_EDITABLE, COL_EXP_N};
+
 #define PREFERENCES(x)	((Preferences *)(x))
 
 
@@ -82,6 +85,12 @@ public:
 	class Biff * biff (void) 				{return biff_;}
 	class Mailbox *added (void)				{return added_;}
 	void added (class Mailbox *mailbox)		{added_ = mailbox;}
+	class Mailbox *selected (void)			{return selected_;}
+	void selected (class Mailbox *mailbox)	{
+		selected_ = mailbox;
+		if (properties_)
+			properties_->select (mailbox);
+	}
 
 	/**
 	 * Callbacks
@@ -91,16 +100,44 @@ public:
 	void on_properties			(GtkWidget *widget);
 	void on_stop				(GtkWidget *widget);
 	void on_close				(GtkWidget *widget);
-	void on_browse_sound		(GtkWidget *widget);
-	void on_test_sound			(GtkWidget *widget);
-	void on_browse_mailapp		(GtkWidget *widget);
 	void on_browse_newmail_image(GtkWidget *widget);
 	void on_browse_nomail_image	(GtkWidget *widget);
 	void on_selection			(GtkTreeSelection *selection);
+	void on_check_changed		(GtkWidget *widget);
 	gboolean on_destroy			(GtkWidget *widget,
 								 GdkEvent *event);
 	gboolean on_delete			(GtkWidget *widget,
-								 GdkEvent *event);	
+								 GdkEvent *event);
+
+	/**
+	 * Expert dialog
+	 **/
+	void expert_create (void);
+	void expert_add_option_list (void);
+	void expert_toggle_option (void);
+	void expert_update_option_list (void);
+	void expert_edit_value (void);
+	void expert_update_option (const gchar *name, class Options *options,
+							   GtkTreeIter *iter);
+	void expert_set_selected_option (const gchar *new_text);
+	void expert_on_selection (GtkTreeSelection *selection);
+	gboolean expert_show_context_menu (GdkEventButton *event);
+	void expert_search (void);
+protected:
+	gboolean expert_get_option (class Options *&options,class Option *&option);
+	gboolean expert_get_option (class Options *&options, class Option *&option,
+								GtkTreeIter &treeiter);
+
+	// Treeview widget for the expert option editing dialog
+	GtkTreeView *expert_treeview;
+	// Liststore for the options in the expert option editing dialog
+	GtkListStore *expert_liststore;
+	// Treeview column that contains the values in the expert option dialog
+	GtkTreeViewColumn *expert_col_value;
+	// Textview for displaying the description of the currently selected option
+	GtkTextView *expert_textview;
+	// Buffer of the description of the currently selected option
+	GtkTextBuffer *expert_textbuffer;
 };
 
 #endif
